local addon = MissionMinder;
addon.name = "MissionMinder"; -- internal name; purposefully non-localized
addon.overriden = {};
addon.hooks = {};
local hooks = addon.hooks;

local L = addon.Localization;


--[[
	C_Garrison.GetInProgressMissions()	-> Returns array of inprogress misssions. Completed missions included only if base xp has yet to be rewarded.
	C_Garrison.MarkMissionComplete()		-> Generates a GARRISON_MISSION_COMPLETE_REPSONSE event
	C_Garrison.MissionBonusRoll()			-> Generates a GARRISON_MISSION_BONUS_ROLL_COMPLETE event
]]--

addon.DebugEnabled = GetAddOnMetadata(addon.name, "X-Debug")

function addon.debug(msg, ...)
	if ( addon.DebugEnabled ) then
		print(string.format(L["|cffff8000<MissionMinder>|cffffffff"].." " .. msg, ...));
	end
end

local DEBUG = addon.debug;

function addon.init()
	-- Auto Open Rewards
	addon.listview.initialize();
	addon.skipfights.initialize();
end


local frame = CreateFrame("Frame");

local events = {
	["ADDON_LOADED"] = function(event, addonName)
		if addonName == addon.name then
			addon:init();
			frame:UnregisterEvent("ADDON_LOADED");
		end
	end,
	["GARRISON_MISSION_COMPLETE_RESPONSE"] = function(event, missionID, canComplete, succeeded)
		local missionLink = C_Garrison.GetMissionLink(missionID);
		print(string.format("Mission %s %s", missionLink, (succeeded and "was successful" or "failed")));

		if ( addon.QuickLoot() and C_Garrison.CanOpenMissionChest(missionID)) then
			C_Garrison.MissionBonusRoll(missionID);
		end
	end,
	["GARRISON_MISSION_BONUS_ROLL_COMPLETE"] = function(event, ...)
		-- Nothing to do
	end,
	["GARRISON_FOLLOWER_XP_CHANGED"] = function(event, followerID, xpAward, oldXP, oldLevel, oldQuality)
		local follower =  C_Garrison.GetFollowerInfo(followerID);
		local followerLink = C_Garrison.GetFollowerLinkByID(follower.garrFollowerID) or follower.name;
		local maxXP = GarrisonMissionFrame_GetFollowerNextLevelXP(oldLevel, oldQuality);

		if ( maxXP ) then  -- nil if already maxLevel and maxQuality
			if ( xpAward + oldXP >= maxXP ) then -- level up
				if ( oldLevel < GarrisonMissionFrame.followerMaxLevel ) then
					print(string.format("Follower %s has reached level %d!", followerLink, oldLevel + 1));
				elseif ( oldQuality < GarrisonMissionFrame.followerMaxQuality ) then
					print(string.format("Follower %s has reached %s quality!", followerLink, getglobal("ITEM_QUALITY" .. oldQuality + 1 .. "_DESC")));
				end
			else
				DEBUG("Follower %s received %d XP (%.1f%%)", followerLink, xpAward, (oldXP + xpAward) / maxXP * 100.0);
			end
		end
	end,
	["GARRISON_MISSION_FINISHED"] = function(event, ...)

	end
};

function frame:OnEvent(event, ...)
	local args = {...}
	local argstr = "";
	for i=1,#args do
		argstr = argstr .. string.format(", arg%d=%s", i, tostring(args[i]));
	end
	DEBUG("OnEvent(event=\"%s\"%s)", event, argstr);

	if ( events[event] ) then
		events[event](event, ...);
	end
end
frame:SetScript("OnEvent", frame.OnEvent);
frame:RegisterEvent("ADDON_LOADED");
frame:RegisterEvent("GARRISON_MISSION_COMPLETE_RESPONSE");
frame:RegisterEvent("GARRISON_MISSION_BONUS_ROLL_COMPLETE");
frame:RegisterEvent("GARRISON_FOLLOWER_XP_CHANGED");
frame:RegisterEvent("GARRISON_MISSION_FINISHED");


function dumpTable(table, indent)
	indent = indent or 0;

	local value = "{";
	for k,v in pairs(table) do
		value = value .. "\n" .. string.rep(" ", indent * 4) .. "\"" .. k .. "\" = ";

		if(type(v) == "table") then
			value = value .. dumpTable(v, indent + 1) .. ",";
		elseif(type(v) == "string") then
			value = value .. "\"" .. v .. "\",";
		else
			value = value .. tostring(v) .. ",";
		end
	end
	indent = indent - 1;
	value = value .. "\n" .. string.rep(" ", indent * 4) .. "}";
	return value;
end