local addon = MissionMinder;
local L = addon.Localization;

local DEBUG_ENABLED = "Debug";
local ENABLE_LIST_VIEW = "EnableListView";
local AUTO_OPEN_REWARDS = "AutoOpenRewards";
local AUTO_CLOSE = "AutoClose";
local SKIP_FIGHT_ANIMATIONS = "SkipFightAnimations";
local QUICK_LOOT = "QuickLoot";

MissionMinderOptions = {};
local MissionMinderOptions_DEFAULT = {
	[DEBUG_ENABLED] = false,
	[ENABLE_LIST_VIEW] = true,
	[AUTO_OPEN_REWARDS] = false,
	[AUTO_CLOSE] = false,
	[SKIP_FIGHT_ANIMATIONS] = false,
	[QUICK_LOOT] = false, -- QUICK_LOOT requires ENABLED_LIST_VIEW
};

function addon.ListViewEnabled()
	return MissionMinderOptions[ENABLE_LIST_VIEW];
end
function addon.AutoOpenRewards()
	return MissionMinderOptions[AUTO_OPEN_REWARDS];
end
function addon.AutoClose()
	return MissionMinderOptions[AUTO_CLOSE];
end
function addon.SkipFightAnimations()
	return MissionMinderOptions[SKIP_FIGHT_ANIMATIONS];
end
function addon.QuickLoot()
	return MissionMinderOptions[QUICK_LOOT];
end

local config = LibStub("AceConfig-3.0")
local dialog = LibStub("AceConfigDialog-3.0")

function createOptions()
	local function getValue(info)
		return MissionMinderOptions[info.arg];
	end

	local function setValue(info, value)
		MissionMinderOptions[info.arg] = value;
	end

	local options = {
		name = L["MissionMinder"] .. " v" .. GetAddOnMetadata(addon.name, "Version"),
		desc = "Manage garrison missions in a more streamline, less-invasive manner",
		type = "group",
		args = {
			desc = {
				order = 2,
				type = "description",
				name = L["Manage garrison missions in a more streamline, less-invasive manner"],
				width = "full"
			},
			emptySpace = {
				order = 3,
				type = "description",
				name = " ",
				width = "full"
			},
			debug = {
				order = 8,
				name = L["Debug"],
				desc = L["Enable debugging"],
				type = "toggle",
				width = "double",
				set = setValue,
				get = getValue,
				arg = DEBUG_ENABLED
			},
			general = {
				order = 9,
				type = "header",
				name = L["General Options"],
			},
			enabled = {
				order = 10,
				name = L["Enable List View"],
				desc = L["View completed missions in the \"In Progress\" tab, instead of sequentially through the completion frame"],
				type = "toggle",
				width = "double",
				set = setValue,
				get = getValue,
				arg = ENABLE_LIST_VIEW
			},
			autoOpenRewards = {
				order = 15,
				name = L["Automatically open rewards |cffcc0000(not yet implemented)"],
				desc = L["Remove the need to click on the rewards chest to recieve the rewards"],
				type = "toggle",
				disabled = true,
				width = "full",
				set = setValue,
				get = getValue,
				arg = AUTO_OPEN_REWARDS
			},
			autoClose = {
				order = 15,
				name = L["Automatically close completion dialog |cffcc0000(not yet implemented)"],
				desc = L["Automatically close the completion dialog if all actions have been completed for a mission"],
				type = "toggle",
				disabled = true,
				width = "full",
				set = setValue,
				get = getValue,
				arg = AUTO_CLOSE
			},
			skipFightAnimations = {
				order = 20,
				name = L["Skip fight animations"],
				desc = L["Skip all fight animations and go straight to rewards"],
				type = "toggle",
				width = "double",
				set = setValue,
				get = getValue,
				arg = SKIP_FIGHT_ANIMATIONS
			},
			quickLoot = {
				order = 25,
				name = L["Quick loot"],
				desc = L["Disable the completion frame entirely, skip all the fanfare, and just give me the loot already!"],
				type = "toggle",
				width = "double",
				set = setValue,
				get = getValue,
				arg = QUICK_LOOT
			}
		}
	};

	-- General Options
	config:RegisterOptionsTable("MissionMinderOptions", options);
	return dialog:AddToBlizOptions("MissionMinderOptions", L["MissionMinder"]);
end


local frame = createOptions();
addon.options = {};
addon.options.frame = frame;


function frame.OnEvent(self, event, arg1)
	if event == "ADDON_LOADED" and arg1 == addon.name then
		-- SavedVariables have been loaded, rewrite to include any new or missing values
		for key,value in pairs(MissionMinderOptions_DEFAULT) do
			if MissionMinderOptions[key] == nil then
				MissionMinderOptions[key] = value;
			end
		end

		-- Rewrite to remove any deprecated settings
		for key,value in pairs(MissionMinderOptions) do
			if MissionMinderOptions_DEFAULT[key] == nil then
				MissionMinderOptions[key] = nil;
			end
		end
 	end
end
frame:SetScript("OnEvent", frame.OnEvent);
frame:RegisterEvent("ADDON_LOADED");
