local addon = MissionMinder;
addon.listview = {};
local hooks = addon.hooks;
local listview = addon.listview;

local L = addon.Localization;
local DEBUG = addon.debug;

function listview.initialize()
	-- "View Completed Missions" button will open "In Progress" tab instead of "CompleteMission" frame
	listview.GarrisonMissionFrame_ShowCompleteMissions = GarrisonMissionFrame_ShowCompleteMissions;
	GarrisonMissionFrame_ShowCompleteMissions = listview.showInProgressMissionList;
	local ViewButton = GarrisonMissionFrame.MissionTab.MissionList.CompleteDialog.BorderFrame.ViewButton;
	ViewButton:SetScript("OnClick", GarrisonMissionFrame_ShowCompleteMissions);

	-- Completed Missions that haven't been looted are not considered "in progress" anymore
	--and need to be added to the list
	listview.GarrisonMissionList_UpdateMissions = GarrisonMissionList_UpdateMissions;
	listview.GarrisonMissionFrame_OnUpdate = GarrisonMissionFrame_OnUpdate;
	GarrisonMissionList_UpdateMissions = listview.missionListUpdate_IncludeCompletedMissions;
	GarrisonMissionFrame_OnUpdate = listview.missionFrameUpdate_IncludeCompletedMissions;

	-- Sort In Progress missions by completed first
	addon.overriden.Garrison_SortMissions= Garrison_SortMissions;
	Garrison_SortMissions = hooks.Garrison_SortMissions;

	-- Allow completed "In Progress" missions to be interactable
	addon.overriden.GarrisonMissionList_Update = GarrisonMissionList_Update;
	GarrisonMissionList_Update = hooks.GarrisonMissionList_Update;

	-- Alter in progress, but completed mission tooltips
	-- OnClick "in progress" missions
	addon.overriden.GarrisonMissionButton_OnClick = GarrisonMissionButton_OnClick;
	addon.overriden.GarrisonMissionButton_OnEnter = GarrisonMissionButton_OnEnter;
	GarrisonMissionButton_OnClick = hooks.GarrisonMissionButton_OnClick;
	GarrisonMissionButton_OnEnter = hooks.GarrisonMissionButton_OnEnter;
	local buttons = GarrisonMissionFrame.MissionTab.MissionList.listScroll.buttons;
	local numButtons = #buttons;
	for i = 1, numButtons do
		local button = buttons[i];
		button:HookScript("OnClick", hooks.GarrisonMissionButton_OnClick);
		button:HookScript("OnEnter", hooks.GarrisonMissionButton_OnEnter);
	end

	-- Mission finished event should not show complete dialog if "In Progress" tab already selected
	addon.overriden.GarrisonMissionFrame_CheckCompleteMissions = GarrisonMissionFrame_CheckCompleteMissions;
	GarrisonMissionFrame_CheckCompleteMissions = hooks.GarrisonMissionFrame_CheckCompleteMissions;

	-- Closing GarrisonMissionFrame should select the "Available" tab. This will cause the next open to show
	--the completed missions dialog again (in applicable)
	GarrisonMissionFrame:HookScript("OnHide", function(...)
		GarrisonMissionList_SetTab(GarrisonMissionFrame.MissionTab.MissionList.Tab1);
	end);

	-- If ListViewEnabled, rewrite "Next" as "Close"
	local MissionComplete = GarrisonMissionFrame.MissionComplete;
	local NextMissionButton = MissionComplete.NextMissionButton;
	addon.overriden.GarrisonMissionCompleteNextButton_OnClick = GarrisonMissionCompleteNextButton_OnClick;
	GarrisonMissionCompleteNextButton_OnClick = hooks.GarrisonMissionCompleteNextButton_OnClick;
	NextMissionButton:SetScript("OnClick", GarrisonMissionCompleteNextButton_OnClick);
	MissionComplete:HookScript("OnShow", listview.updateNextMissionButtonText);

	-- FIX preload model defect
	listview.MissionCompletePreload_LoadMission = MissionCompletePreload_LoadMission;
	MissionCompletePreload_LoadMission = listview.PreloadMissionModelsFIX;
end



-- Alter sort to have completed missions at the top
function hooks.Garrison_SortMissions(missionsList)
	if ( addon.ListViewEnabled() ) then
		table.sort(missionsList, addon.SortMissionsComparator);
	else
		addon.overriden.Garrison_SortMissions(missionsList);
	end
end

-- Create tooltip for in progress misisons
function hooks.GarrisonMissionButton_OnEnter(...)
	local self, button = ...
	local mission = self.info;
	if (mission.inProgress) then
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR_RIGHT");

		--Mission Name
		GameTooltip:SetText(mission.name);
		if ( addon.IsMissionComplete(mission) ) then
			GameTooltip:AddLine(L["|cff00ff00(Complete)"], 1, 1, 1);
		else
			GameTooltip:AddLine(self.info.timeLeft.." "..RED_FONT_COLOR_CODE..GARRISON_MISSION_IN_PROGRESS..FONT_COLOR_CODE_CLOSE, 1, 1, 1);
		end

		GameTooltip:AddLine(" ");
		if mission.followers ~= nil then
			GameTooltip:AddLine(GARRISON_FOLLOWERS);
			for i=1, #(mission.followers) do
				GameTooltip:AddLine(C_Garrison.GetFollowerName(mission.followers[i]), 1, 1, 1);
			end
		end

		GameTooltip:Show();
	else
		addon.overriden.GarrisonMissionButton_OnEnter(...);
	end

end


function listview.showInProgressMissionList(...)
	if (not addon.ListViewEnabled()) then
		addon.overriden:GarrisonMissionFrame_ShowCompleteMissions(...);
		return;
	end

	PlaySound("UI_Garrison_CommandTable_ViewMissionReport");
	GarrisonMissionList_SetTab(GarrisonMissionFrame.MissionTab.MissionList.Tab2);
	GarrisonMissionFrame.MissionTab.MissionList.CompleteDialog:Hide();
end


function listview.missionListUpdate_IncludeCompletedMissions(...)
	local self = GarrisonMissionFrame.MissionTab.MissionList;
	addon.GetInProgressAndCompletedMissions(self.inProgressMissions);
	C_Garrison.GetAvailableMissions(self.availableMissions);
	Garrison_SortMissions(self.availableMissions);
	self.Tab1:SetText(AVAILABLE.." - "..#self.availableMissions)
	self.Tab2:SetText(WINTERGRASP_IN_PROGRESS.." - "..#self.inProgressMissions)
	if ( #self.inProgressMissions > 0 ) then
		self.Tab2.Left:SetDesaturated(false);
		self.Tab2.Right:SetDesaturated(false);
		self.Tab2.Middle:SetDesaturated(false);
		self.Tab2.Text:SetTextColor(1, 1, 1);
		self.Tab2:SetEnabled(true);
	else
		self.Tab2.Left:SetDesaturated(true);
		self.Tab2.Right:SetDesaturated(true);
		self.Tab2.Middle:SetDesaturated(true);
		self.Tab2.Text:SetTextColor(0.5, 0.5, 0.5);
		self.Tab2:SetEnabled(false);
	end
	GarrisonMissionList_Update();
end


function listview.missionFrameUpdate_IncludeCompletedMissions(...)
	local self = GarrisonMissionFrame.MissionTab.MissionList;
	if (self.showInProgress) then
		addon.GetInProgressAndCompletedMissions(self.inProgressMissions);
		self.Tab2:SetText(WINTERGRASP_IN_PROGRESS.." - "..#self.inProgressMissions)

		if( #self.inProgressMissions == 0) then
			self:SetScript("OnUpdate", nil);
		end
	else
		self.availableMissions = C_Garrison.GetAvailableMissions();
		Garrison_SortMissions(self.availableMissions);
		self.Tab1:SetText(AVAILABLE.." - "..#self.availableMissions)
		self:SetScript("OnUpdate", nil);
	end
	GarrisonMissionList_Update();
end


-- When user clicks on a mission from the mission list (either available or in progress)
function hooks.GarrisonMissionButton_OnClick(...)
	local self, button = ...

	-- Ignore CHATLINK attempts and non-inProgress missions (handled properly already)
	if ( IsModifiedClick("CHATLINK") or not self.info.inProgress ) then
		return
	end

	if ( addon.ListViewEnabled() ) then
		local mission = self.info;
		if ( addon.IsMissionComplete(mission) ) then -- Mission must be completed
			DEBUG("Loading mission [%d] %s", mission.missionID, mission.name);
			if ( addon.QuickLoot() ) then
				if ( mission.state >= 0 ) then
					if ( C_Garrison.CanOpenMissionChest(mission.missionID) ) then
						C_Garrison.MissionBonusRoll(mission.missionID);
					end
				else
					C_Garrison.MarkMissionComplete(mission.missionID);
				end
			else

				-- This would be part of the OnClick event of the In Progress tab
				GarrisonMissionFrame.FollowerTab:Hide();
				GarrisonMissionFrame.FollowerList:Hide();
				HelpPlate_Hide();

				GarrisonMissionFrame.MissionComplete:Show();
				GarrisonMissionFrame.MissionCompleteBackground:Show();

				local self = GarrisonMissionFrame.MissionComplete;
				self.currentIndex = addon.IndexOfMission(self.completeMissions, mission.missionID);
				if ( self.currentIndex == 0 ) then -- We may not find the mission in the list, reload completeMission list
					self.completeMissions = C_Garrison.GetCompleteMissions();
					self.currentIndex = addon.IndexOfMission(self.completeMissions, mission.missionID);
				end

				MissionCompletePreload_Cancel(); -- Cancel any previous preload that is still running
				MissionCompletePreload_LoadMission(mission.missionID); -- Start preload
				GarrisonMissionComplete_Initialize(self.completeMissions, self.currentIndex);
				return;
			end
		end
	end
end


function hooks.GarrisonMissionList_Update(...)
	addon.overriden.GarrisonMissionList_Update(...);

	-- Hide overlay on completed missions
	local self = GarrisonMissionFrame.MissionTab.MissionList;

	local missions = (self.showInProgress) and self.inProgressMissions or self.availableMissions;
	local numMissions = #missions;
	local scrollFrame = self.listScroll;
	local offset = HybridScrollFrame_GetOffset(scrollFrame);
	local buttons = scrollFrame.buttons;
	local numButtons = #buttons;

	for i = 1, numButtons do
		local button = buttons[i];
		local index = offset + i; -- adjust index

		if ( index <= numMissions) then
			local mission = missions[index];
			if( self.showInProgress and addon.IsMissionComplete(mission) and addon.ListViewEnabled() ) then
				button.Overlay:Hide();
				button.LocBG:SetVertexColor(0.3, 1, 1, 1);
				button.Summary:SetText(L["|cff00ff00(Complete)"]);
			else
				button.LocBG:SetVertexColor(1, 1, 1, 1);
			end
		end
	end
end


function hooks.GarrisonMissionFrame_CheckCompleteMissions(...)
	-- Do not popup mission complete dialog if the "In Progress" mission tab is already selected
	if ( addon.ListViewEnabled() ) then
		local self = GarrisonMissionFrame;
		if( self:IsShown() and self.MissionTab.MissionList.showInProgress) then
			return;
		end
	end

	addon.overriden.GarrisonMissionFrame_CheckCompleteMissions(...);
end


function hooks.GarrisonMissionCompleteNextButton_OnClick(...)
	if ( not addon.ListViewEnabled() ) then
		addon.overriden.GarrisonMissionCompleteNextButton_OnClick(...);
		return;
	end

	GarrisonMissionFrame_HideCompleteMissions(true);
end






MISSION_STATE = {
	["AVAILABLE"] = -2,
	["IN_PROGRESS"] = -1,
	["COMPLETED"] = 0
};

function addon.GetInProgressAndCompletedMissions(missions)
	C_Garrison.GetInProgressMissions(missions);

	if ( addon.ListViewEnabled() ) then
		local completed = C_Garrison.GetCompleteMissions();
		-- Add completed mission to list
		for index = 1, #completed do
			local mission = completed[index];
			if ( mission.state == MISSION_STATE.COMPLETED ) then
				-- label this mission as in progress
				mission.inProgress = true;
				mission.timeLeft = "0 sec";
				missions[#missions + 1] = mission;
			end
		end

		Garrison_SortMissions(missions);
	end
end

function addon.IndexOfMission(missions, missionID)
	for key,value in pairs(missions) do -- find this mission in the completedMissions list
		if ( value.missionID == missionID ) then
			DEBUG("IndexOfMissions(%d) => [%d] %s", missionID, value.missionID, value.name);
			return key;
		end
	end

	return 0;
end

function addon.IsMissionComplete(mission)
	return mission.state == MISSION_STATE.COMPLETED or mission.timeLeft == "0 sec";
end

function addon.SortMissionsComparator(mission1, mission2)
	local mission1Complete = addon.IsMissionComplete(mission1);
	local mission2Complete = addon.IsMissionComplete(mission2);
	if ( mission1Complete ~= mission2Complete ) then
		return mission1Complete;
	end

	-- Blizzard code below --
	if ( mission1.level ~= mission2.level ) then
		return mission1.level > mission2.level;
	end

	if ( mission1.level == GARRISON_FOLLOWER_MAX_LEVEL ) then	-- mission 2 level is same as 1's at this point
		if ( mission1.iLevel ~= mission2.iLevel ) then
			return mission1.iLevel > mission2.iLevel;
		end
	end

	if ( mission1.durationSeconds ~= mission2.durationSeconds ) then
		return mission1.durationSeconds < mission2.durationSeconds;
	end

	if ( mission1.isRare ~= mission2.isRare ) then
		return mission1.isRare;
	end

	return strcmputf8i(mission1.name, mission2.name) < 0;
end

local nextString = nil;


function listview.updateNextMissionButtonText(...)
	local NextMissionButton = GarrisonMissionFrame.MissionComplete.NextMissionButton;

	-- Retrieve the original next string (allows for blizzard to localize the string for us)
	nextString = nextString or NextMissionButton:GetText();

	if ( addon.ListViewEnabled() ) then
		NextMissionButton:SetText(L["Close"]);
	else
		NextMissionButton:SetText(nextString);
	end
end


-- Had to be altered to handle scenario where C_Garrison.GetMissionDisplayIDs(missionID) returns nil.
--This case seems to occur when models have already been preloaded, and we try to preload a second time.
function listview.PreloadMissionModelsFIX(...)
	if ( addon.ListViewEnabled() ) then
		local missionID = ...;
		if ( C_Garrison.GetMissionDisplayIDs(missionID) == nil ) then
			return; -- If no values returned, then return without preloading
		end
	end

	listview.MissionCompletePreload_LoadMission(...);
end