local addon = MissionMinder;
addon.skipfights = {};
local skipfights = addon.skipfights;

local L = addon.Localization;
local DEBUG = addon.debug;

local SKIP_ANIMATION_CONTROL = nil;


skipfights.ANIMATION = {
	[1] = "LINE",
	[2] = "MODELS_LOADED_CHECK",
	[3] = "MODEL_FIGHT",
	[4] = "IMPACT_SOUND",
	[5] = "X_OVER_PORTRAIT",
	[6] = "NEXT_ENCOUNTER",
	[7] = "REWARD_PANEL",
	[8] = "EXPLODE_LOCK",
	[9] = "SHOW_FOLLOWERS",
	[10] = "FOLLOWER_XP"
};


function skipfights.initialize()
	SKIP_ANIMATION_CONTROL = {
		[1] = addon.SkipFightAnimations,	-- line between encounters
		[2] = addon.SkipFightAnimations,	-- check that models are loaded
		[3] = addon.SkipFightAnimations,	-- model fight
		[4] = addon.SkipFightAnimations,	-- impact sound when follower hits
		[5] = addon.SkipFightAnimations,	-- X over portrait
		[6] = addon.SkipFightAnimations,	-- evaluate whether to do next encounter or move on
		[7] = nil,				-- reward panel
		[8] = nil,				-- explode the lock if mission successful
		[9] = nil,				-- show all the mission followers
		[10] = nil				-- follower xp
	};

	-- Skip fight animations
	skipfights.GarrisonMissionComplete_OnUpdate = GarrisonMissionComplete_OnUpdate;
	GarrisonMissionComplete_OnUpdate = skipfights.skipAnimations;
	if(GarrisonMissionFrame.MissionComplete:GetScript("OnUpdate")) then
		GarrisonMissionFrame.MissionComplete:SetScript("OnUpdate", GarrisonMissionComplete_OnUpdate);
	end

	-- Hide encounter frame (enemy unit frames)
	hooksecurefunc("GarrisonMissionComplete_Initialize", skipfights.hideEncounterFrame);

	-- Show the success/failure message
	hooksecurefunc("GarrisonMissionComplete_OnMissionCompleteResponse", skipfights.showSuccessFailureMessage);
end


-- Skip any fight animations and sound as defind in SKIP_ANIMATION_CONTROL
function skipfights.skipAnimations(...)
	if ( addon.SkipFightAnimations() ) then
		local self, elapsed = ...

		local skipControl = SKIP_ANIMATION_CONTROL[self.animIndex];

		while ( skipControl and skipControl() ) do
			DEBUG("Skipping animation [%d] %s", self.animIndex, skipfights.ANIMATION[self.animIndex]);

			self.animIndex = self.animIndex + 1;
			self.animTimeLeft = 0;
			skipControl = SKIP_ANIMATION_CONTROL[self.animIndex];
		end
	end

	skipfights.GarrisonMissionComplete_OnUpdate(...);
end


-- if SkipFightAnimations, hide enemy unit frames from completion frame
function skipfights.hideEncounterFrame(...)
	if ( addon.SkipFightAnimations() ) then
		GarrisonMissionFrame.MissionComplete.Stage.EncountersFrame:Hide();
	end
end


-- Show mission success/failure message
function skipfights.showSuccessFailureMessage(self, missionID, canComplete, succeeded)
	if( addon.SkipFightAnimations() ) then
		local self = GarrisonMissionFrame.MissionComplete.ChanceFrame;
		self.ChanceText:SetAlpha(0)
		if (succeeded) then
			self.SuccessAnim:Play();
			self.SuccessText:SetAlpha(1);
			self.FailureText:SetAlpha(0);
			PlaySound("UI_Garrison_CommandTable_MissionSuccess_Stinger");
		else
			self.FailureAnim:Play();
			self.SuccessText:SetAlpha(0);
			self.FailureText:SetAlpha(1);
		PlaySound("UI_Garrison_Mission_Complete_MissionFail_Stinger");
		end
	end
end